/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (C) 2002   SEGGER Microcontroller Systeme GmbH               *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for ST7)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include <iodefine.h>
#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

#ifndef OS_FSYS
  #define OS_FSYS 8000000L  /* internal peripheral clock frequency  */
#endif

/* Select UART for embOSView.
   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define it to be -1
   Currently the standard code below supports SCI (named  UART 0)
*/

#ifndef OS_UART
  #define OS_UART 0
#endif    
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware timer required for embOS. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if you have to use an other timer.
*/

#define PRESCALE 32
#define PRESCALEVALUE 0x50 // Register Value for Prescaler
#define CYCLES_PER_TICK    (OS_FSYS/1000/PRESCALE)

void OS_InitHW(void) {
  ARTCSR = 0x00;           // Disable ART Timer and interrupt
  ARTCSR = PRESCALEVALUE;  // prescaler set to div 32; count and int disable
  ARTARR = 256-CYCLES_PER_TICK;      // Generate Interrupts every 1 ms
  ISPR3  |= 0x0C;          // Set Timer interrupt priority 3
  ARTCSR |= (1<<2);        // Force reload, reset interrupt pending
  ARTCSR = PRESCALEVALUE | 0x0A;     // Start timer, enable interrupt
  OS_COM_Init();           // Initialize UART for embOSView
  OS_EI();                 // Enable interrupts now
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not run as task, so it does not have have a stack of its own,
 therefore no functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {       // Idle loop: No task is ready to exec
  for (;;);                // Nothing to do ... wait for interrupt
}

/*********************************************************************
*
*       Run-time error reaction  (OS_Error)
*
**********************************************************************

 Run-time error reaction

 When this happens, a fatal error has occured and the kernel
 can not continue.
 
 This routine can be modified to suit your needs ...
 E.g. a red LED could light up. When using an
 emulator, you may set a breakpoint here.
 In the release builds of the library (R), this routine is not required
 (as no checks are performed).
 In the stack check builds (S/SP), only error 120 may occur.
 In the debug builds(D/DP), all of the listed errors may occur.

 The following are the current errorcodes:
 (Please refer to the documentation for more info)
  
 *** Stack overflow ***  
 OS_ERR_STACK                          (120 = 0x78)

 *** invalid or non-initialized data structures ***
 OS_ERR_INV_TASK                       (128 = 0x80)
 OS_ERR_INV_TIMER                      (129 = 0x81)
 OS_ERR_INV_MAILBOX                    (130 = 0x82)
 OS_ERR_INV_CSEMA                      (132 = 0x84)
 OS_ERR_INV_RSEMA                      (133 = 0x85)

 *** Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on a non-1 byte mailbox  ***
 OS_ERR_MAILBOX_NOT1                   (135 = 0x87)

 ***internal errors, please contact SEGGER Microcontrollersysteme ***
 OS_ERR_MAILBOX_NOT_IN_LIST            (140 = 0x8C)
 OS_ERR_TASKLIST_CORRUPT               (142 = 0x8E)

 *** not matching routine calls or macro usage ***
 OS_ERR_UNUSE_BEFORE_USE               (150 = 0x96)
 OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151 = 0x97)
 OS_ERR_LEAVEINT                       (152 = 0x98)
 OS_ERR_DICNT                          (153 = 0x99)
 OS_ERR_INTERRUPT_DISABLED             (154 = 0x9A)

 *** not a legal system call during interrupt ***
 OS_ERR_ILLEGAL_IN_ISR                 (160 = 0xA0)
 
 *** not a legal system call during timer ***
 OS_ERR_ILLEGAL_IN_TIMER               (161 = 0xA1)

 *** double used data structures ***
 OS_ERR_2USE_TASK                      (170 = 0xAA)
 OS_ERR_2USE_TIMER                     (171 = 0xAB)
 OS_ERR_2USE_MAILBOX                   (172 = 0xAC)
 OS_ERR_2USE_BSEMA                     (173 = 0xAD)
 OS_ERR_2USE_CSEMA                     (174 = 0xAE)
 OS_ERR_2USE_RSEMA                     (175 = 0xAF)

*/

void OS_Error(int ErrCode) {
  OS_EnterRegion();     /* Avoid further task switches */
  OS_DICnt  = 0;        /* Allow interrupts so we can communicate */
  OS_EI();
  OS_Status = ErrCode;
  while (OS_Status) {
  }  
}

/*********************************************************************
*
*       OS_GetTime_Cyles
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in timer clock cycles.
    This routine should not be called by user and
    must only be called with interrupts disabled.
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = 256-ARTCAR;
  OS_U32 time = OS_Time + 1;
  // Check if timer interrupt pending ...
  if (ARTCSR & (1<<0)) {   // This unfortunately resets interrupt pending flag !!!
    t_cnt = 256-ARTCAR;
    time++;
    OS_TickHandlerDI();    // Call special tick handler, because we miss a tick !!!
  }
  return CYCLES_PER_TICK*time - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

   Convert timer cycles into micro seconds. When using 20.0 MHz, this
   means dividing by 20. If you have a different clock frequency,
   you may have to modify this routine in order to get proper
   diagonstics. Please note: Operation of theOS is not affected
   by this routine. It is used for profiling and testing
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= PRESCALE;
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
*/

@interrupt @svlreg void OS_ISR_Tick (void) { 
  ARTCSR |= (0);   // reset interrupt pending(read CSR)
  OS_TickHandler();
}

/*********************************************************************
*
*       Communication for embOSView   (UART 0)
*
**********************************************************************
*/

#define BAUDDIVIDE OS_FSYS/13/16/OS_BAUDRATE

#if (OS_UART == 0)
  #if BAUDDIVIDE == 1
    #define BAUDTR 0
  #elif BAUDDIVIDE == 2
    #define BAUDTR 1
  #elif BAUDDIVIDE == 4
    #define BAUDTR 2
  #else
    #error "Baudrate / CPU clock combination not supported"
  #endif

void OS_COM_Init(void) {
  ISPR2  |= 0x30;         //  lock SCI interrupt
  SCICR2  = 0x00;         //  lock Rx Tx, interrupts
  SCICR1  = 0x00;         //  Data length 8 bit, no parity, no parity int.
  SCIERPR = 0x00;         //  Extended Baudrate generator not used
  SCIETPR = 0x00;         //  Extended Baudrate generator not used
  SCIBRR  = 0xC0          //  Setup Baudrate generator, Prescaler fixed to 13
          | BAUDTR<<3     //  Set Tx Baudrate
          | BAUDTR;       //  Set Rx Baudrate to same value
  SCICR2  =(0<<0)         //  SBK  bit: Do not send Break
          |(0<<1)         //  RWU  bit: Receiver in active mode
          |(1<<2)         //  RE   bit: Rx enable
          |(1<<3)         //  TE   bit: Tx Enable
          |(0<<4)         //  ILIE bit: Idle line interrupt disable
          |(1<<5)         //  RIE  bit: Rx Interrupt enable
          |(0<<6)         //  TCIE bit: Tx Complete interrupt disable
          |(0<<7);        //  TIE  bit: Tx interrupt initially disabled
  ISPR2  |= 0x30;         //  enable SCISR interrupt level 3
}

void OS_COM_Send1(OS_U8 c) {
  /* Send one byte. NOTE: Never call this routine from your appilcation */
  SCIDR   = c;
  SCICR2 |= (1<<7);       //  TIE  bit: Enable Tx interrupts
}

/* The interrupt routine handles both: Rx and Tx interrupt */

@interrupt @svlreg void OS_ISR_SCI(void) {
  /*  Handles both: Rx and Tx interrupts  */
  OS_U8 Data;
  OS_U8 Status;

//  OS_EnterNestableInterrupt(); //  We will enable interrupts 
  OS_EnterInterrupt(); //  Do not enable interrupts 
  Status = SCISR;            //  get SCI status

  if (Status & 0x0B) {       //  Check if errors occured (Parity, framing, overrun)
    SCICR2 &= ~(1<<2);       //  Disable Rx
    SCISR;                   //  Dummy reads to reset errors
    SCIDR;                   //
    SCICR2 |= (1<<2);        //  ReEnable Rx
  } else {
    if (Status & (1<<5)) {   //  Data received ?
      Data   = SCIDR;        //  Read Data
      OS_OnRx(Data);         //  Process actual data 
    }
  }
  if (SCICR2 & (1<<7)) {     //  Tx interrupt enabled ?
    if (SCISR & (1<<7)) {    //  Tx Empty interrupt ?
      if (OS_OnTx()) {       //  process next data to send 
        SCICR2 &= ~(1<<7);   //  Disable Tx interrupts, if no data were sent
      }
    }   
  }
//  OS_LeaveNestableInterrupt();
  OS_LeaveInterrupt();
}

/*  ----------------------------------------------------------------  */

#else  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  // let OS know that transmitter is not busy
  }

void OS_ISR_SCI(void) {}
  
#endif /* (OS_UART == 0 ) */

